;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x73d6e02ffe8067dfbd9b170e98ed3c9e . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x3ae3360c4e3164e260a0394269414d17 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x97cba7c487fc4f31679cac98b5428bc9 . "./code.cg2js/cg-grid-widget.fasl")
(#x5341dfdea7dce4517f07db9919392403 . "./code.cg2js/cg-utility-dialog.fasl")
(#x5e0234966f214fb900490b416ce2f85e . "./code.cg2js/cg-wrap-string.fasl")
(#x4651e449eed8d1d277942fe353ec572e . "./code.cg2js/cg-editable-text.fasl")
(#x52d19ce8c1fdaccbfa778f9073377e4f . "./code.cg2js/cg-base.fasl")
(#xbdc87e29a15ca04f666808238c4c49ff . "./code.cg2js/cg-lisp-text.fasl")
(#x465ecb23f20d2d8bab9decf1e63af739 . "./code.cg2js/cg-intersect.fasl")
(#xf28544e14c8fbd2cd5abcff018128047 . "./code.cg2js/cg-choose-printer.fasl")
(#x47cd396161613c8ac683b758573c5142 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x798a13e4d66c4b1edc0ef2ccccf761e7 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xc2daf13c75760edad04fdc4d93d4e5a8 . "./code.cg2js/cg-font-dialog.fasl")
(#x02f0ba9392f21efc80aac216ff1ca31a . "./code.cg2js/cg-comtab.fasl")
(#x3301b283790c8fcfcc7eb7b93680f9f7 . "./code.cg2js/cg-tooltip.fasl")
(#xee7d02a4a0b4dcebc940f15832848fc2 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x3c6bc94352c5f546a3e5ef0d15f07414 . "./code.cg2js/cg-get-pixmap.fasl")
(#xb608048d9f1c73b5873f660179f9e2ec . "./code.cg2js/cg-status-bar.fasl")
(#x4be06da7b9185a515c5096ecebe6ba27 . "./code.cg2js/cg-tab-control.fasl")
(#x0b61e035b4e365e7978d7563fcb81c1f . "./code.cg2js/cg-icon-pixmap.fasl")
(#x99f11f40958e39be1aaaba6f94c969fc . "./code.cg2js/cg-list-view.fasl")
(#xc78e408183cbf710c342daaacd39bf0c . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xf1a990a0aa32533a9208c7d76c90a61b . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x3d5b8614d363ac307976886d9bba52d8 . "./code.cg2js/cg-project-window.fasl")
(#x6787b42cc4065b0210f0bb07f73e5ad5 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x8bdc3b2d74e9495800cfb3fb70e2e68f . "./code.cg2js/cg-drag-and-drop.fasl")
(#xf0acef729378b05aae8a790bf18d6e35 . "./code.cg2js/cg-scaling-stream.fasl")
(#xa0f4f37525ba29e202e8f081c760f36c . "./code.cg2js/cg-bitmap-stream.fasl")
(#x237b2ab2dcff3e2bdf1a5f8313f87c06 . "./code.cg2js/cg-menu.fasl")
(#x5518d5b49e2b6c461278172838f3cbbe . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x56f508c66bac60bc1c5ea0a300cf8838 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x471c3aba4009c8289d8c9f1f48197d91 . "./code.cg2js/cg-split-bar.fasl")
(#xfce495e0ed13c8ebdb35e728b78e436f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd1ec1c5cdce03ad234d231a6287de1af . "./code.cg2js/cg-timer.fasl")
(#xfc2ee65d5a95535cc673c15734ead0ed . "./code.cg2js/cg-text-or-combo.fasl")
(#x325a30564b60960a8118a8596859d16d . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x022f30a5ad567e86b210f3fe6007e45a . "./code.cg2js/cg-string-dialog.fasl")
(#x6591f648b14c467b8a760bb3f63150a8 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xd52f3ff858bef3c60e8f63a12230f215 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xaf2982f7650713973d401e6ae98dcf64 . "./code.cg2js/cg-outline.fasl")
(#x3798e7e104b97b59be73c44088084844 . "./code.cg2js/cg-lettered-menu.fasl")
(#x23e1a5d7128b9a282d13a1a621abe7c1 . "./code.cg2js/cg-graphics-context.fasl")
(#x4406b0a47ade97c9d58595c81813c329 . "./code.cg2js/cg-item-list.fasl")
(#x971438adf94bcd38614d47f6b077f7ed . "./code.cg2js/cg-caret.fasl")
(#xa66c7a92ade935311793d64c8d5bb904 . "./code.cg2js/cg-printing.fasl")
(#xbaf3ce02725a3461f14f8a9f61b5d992 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xbba115bdd1aa78853922364258cf7453 . "./code.cg2js/cg-progress-indicator.fasl")
(#x15184daa0dd8ffc86a8d7885d69d346c . "./code.cg2js/cg-dde-utils.fasl")
(#x7a3539090f305fde84fcc9c2f5c3daee . "./code.cg2js/cg-plot-widget.fasl")
(#xbe234e84fed27dd11d067f7d351d8dc3 . "./code.cg2js/cg-os-window.fasl")
(#x4a852bca6377e30c1707ed47364b55f8 . "./code.cg2js/cg-calendar.fasl")
(#x1aac5133d1053048dd7a8d294a0a510d . "./code.cg2js/cg-chart-widget.fasl")
(#xfe0c62badc623588d492964c128b1dbf . "./code.cg2js/cg-file-dialog.fasl")
(#xddb5a5acd2a42abe9b0cdb2b30e1d994 . "./code.cg2js/cg-nodes.fasl")
(#x273a5c10139360ef07161615f6b8f312 . "./code.cg2js/cg-lamp.fasl")
(#x506715fa0b8fec149e95a5784cb48a91 . "./code.cg2js/cg-html-dialog.fasl")
(#x380192e1436f5f9f9ece35eec2caf643 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x3df630ac23eb50a8e815ae67069531b4 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x67ecdfc2a65dc4716340dde461ed3abd . "./code.cg2js/cg-choice-list.fasl")
(#x2d5ee41d348dde96e6fe2b9084f0376e . "./code.cg2js/cg-multi-picture-button.fasl")
(#xa59aa1a566c2cddc8b1dfb01daf2de12 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x1ce858d61911c2bc61fcc09f780c986c . "./code.cg2js/cg-hotspot.fasl")
(#x8805b52b6d069473f51f326e190ec9bf . "./code.cg2js/cg-property.fasl")
(#xa6b7f053e2e33fc96b7652a3ab141af6 . "./code.cg2js/cg-dropping-outline.fasl")
(#x556302c2d7ca421e664f5207178ded93 . "./code.cg2js/cg-template-string.fasl")
(#xc4efa481aa249a153c46bc217919be8b . "./code.cg2js/cg-palette.fasl")
(#xbe77b0d807269753271bb20513b85975 . "./code.cg2js/cg-selected-object.fasl")
(#x7180d369326d6f7d7b1b4808e98d019d . "./code.cg2js/cg-up-down-control.fasl")
(#xa7c355c05f0fbe360e8151c6150f29d0 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xb463622456654344529aecbc329d71e7 . "./code.cg2js/cg-layout.fasl")
(#xfe57e43fda811e7700f2516d2cef994e . "./code.cg2js/cg-get-position.fasl")
(#x28f096d3b8b4dd88a5a8a242df3929de . "./code.cg2js/cg-lisp-widget.fasl")
(#x2394a39b2b8c2f38cda5baa1af6f4d00 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x1d6c1ba74a3321ca07825777b90e7275 . "./code.cg2js/cg-object-editor.fasl")
(#xd7fc70df3f85df35156b12640da7a093 . "./code.cg2js/cg-class-grid.fasl")
(#x20806974057d4c30bfd802bd4be6e09e . "./code.cg2js/cg-yes-no-string.fasl")
(#x3fd17af5ae2c932f6c71a659670b6b40 . "./code.cg2js/cg-toggling-widget.fasl")
(#xc7a653b3affcb538cfe81c2cec38abb7 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x80a2f15f2ad12f5ffe365f8416e001f3 . "./code.cg2js/cg-button.fasl")
(#xb2ab342d92f89b2129edc706da53e91a . "./code.cg2js/cg-object-editor-layout.fasl")
(#x3738b7499e7014abf5425ee982b7f238 . "./code.cg2js/cg-scroll-bar.fasl")
(#x176e9d197671f9cb889b836a895fd4ce . "./code.cg2js/cg-combo-box.fasl")
(#xe64e4a8aa18a0dc50e1c234191ac94c6 . "./code.cg2js/cg-web-browser.fasl")
(#xd439b1804fdf34de43d09ec320f603ec . "./code.cg2js/cg-drawable.fasl")
(#xee8f53268fe7cbc5e79a103474641dd2 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb6b253e04641ed8db9b24a540c43b5a9 . "./code.cg2js/cg-message-dialog.fasl")
(#x2519e11dcff132051b2bc45170d82ec9 . "./code.cg2js/cg-text-widget.fasl")
(#xf36a490a508265c3316d029b176cff0a . "./code.cg2js/cg-static-text.fasl")
(#x1fa894b86b2e0b700290a55116755a36 . "./code.cg2js/cg-class-support.fasl")
(#xaea3dac45d32e7a98c9b6d8f7e6d7654 . "./code.cg2js/cg.fasl")
(#x576dd96e91ae2f095d794cec8d138762 . "./code.cg2js/cg-directory-dialog.fasl")
(#x5d864e60e55435962d5613d99570205c . "./code.cg2js/cg-clipboard.fasl")
(#xe2096ba50f4be6b182812c94c756a14a . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xc7c501f267461bfff90d91c19b66923d . "./code.cg2js/cg-bitmap-pane.fasl")
(#x9ea7e73ad784fdd4cb04407520f3d0a6 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xf4620dcd25019103bb1a770f5d04e280 . "./code.cg2js/cg-check-box.fasl")
(#xfb17815634e45abbafb0e3f75a581449 . "./code.cg2js/cg-color-dialog.fasl")
(#x8e77deed37a4d80ad4c55ad71b08cdaf . "./code.cg2js/cg-text-edit-pane.fasl")
(#x02303ffb30a8aff191d30b4bf799be68 . "./code.cg2js/cg-dialog-item.fasl")
(#xbe44b923eb45111c4f23d4121b8cb336 . "./code.cg2js/cg-edit-in-place.fasl")
(#x9c5e2df86d1fd7ecd824f32867d70b56 . "./code.cg2js/cg-tray.fasl")
(#xe657add68bd074d9d5bd5040215cc99c . "./code.cg2js/cg-paren-matching.fasl")
(#x633c7e70946783b98824fbf236add7f5 . "./code.cg2js/cg-player.fasl")
(#xc3444600727f0d2000ea3467dbfa961c . "./code.cg2js/cg-yes-no-list.fasl")
(#xeb6df20477159b412830af7d301312e5 . "./code.cg2js/cg-common-control.fasl")
(#x5aa8c518802a2b6b0ed9a3bad9e7378d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x716a3ebaf816393498a6973e37d28741 . "./code.cg2js/cg-os-widget.fasl")
(#xd82cc4288bde44e899befa615aea2404 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xbe124c26796124619c5bb1ba609a5cd9 . "./code.cg2js/cg-trackbar.fasl")
(#x4bd74064a40f046af85e4a64526b72c4 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x47c31e7ea80b1a3bb161def5ff3074b1 . "./code.cg2js/cg-html-widget.fasl")
(#x32d5aee4d4a09aeb47e01f2e1286f17c . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xcfcdb8b9c97bfa6bb5fb50e014ee42f5 . "./code.cg2js/cg-acache.fasl")
(#x15b9ca9df2f501365459b77a12db6493 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x641582e25232686ef4b803b1917589b4 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xd3fbf0da116385886527ccf298fe561c . "./code.cg2js/cg-icon.fasl")
(#x533f4247c4102aede588b3982c3f9566 . "./code.cg2js/cg-pixmap.fasl")
(#x7b037327e38af14ec13afad163aa9609 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x186d8e23338759a7f36aec2de1cf6a8b . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x3f283f3d461cbb82693db229a4831060 . "./code.cg2js/cg-toolbar.fasl")
(#xcea66583ea07490b82096785b4ff000c . "./code.cg2js/cg-radio-button.fasl")
(#x866f453f05347405f1f2726f04ad4782 . "./code.cg2js/cg-curve.fasl")
(#x04d49149b56045b41777a5700d2edf09 . "./code.cg2js/cg-picture-widget.fasl")
(#xe25337bbc803d87caed3c2e2a5f02783 . "./code.cg2js/cg-dev.fasl")
(#xfc8778e2ec11f7775f929844b50d6c46 . "./code.cg2js/cg-group-box.fasl")
(#xbb8286097514123a051c0ff9cdb83932 . "./code.cg2js/cg-fill-texture.fasl")
(#x02e32b07335875e951212b15b61bc1d0 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x9e37f9d8e838fb162709efbb6e0dc49d . "./code.idejs/ide-keyshort.fasl")
(#xc1d168a22cfe6a72a5ccebec7d6e362d . "./code.idejs/ide-form.fasl")
(#x8e6318e62ef7c36617baa93bd32711a7 . "./code.idejs/ide-find-in-files.fasl")
(#x6f6f4dcca14e588ca51188124ae98a24 . "./code.idejs/ide-class-grapher.fasl")
(#xbeb523d7878f4664ad9444ee67f589d2 . "./code.idejs/ide-base.fasl")
(#x6b9ab53da07d62e2f8117735df6468f1 . "./code.idejs/ide-class-browser.fasl")
(#xc23aaade8f432f2329e29b8462ebd01d . "./code.idejs/ide-find-definitions.fasl")
(#x01b3d6b2e8fc30eeee1e3c39fa788342 . "./code.idejs/ide-package-list.fasl")
(#xbdb3db4720139134ebe147d18a098246 . "./code.idejs/ide-grapher.fasl")
(#x81d839103542a04cdaf397fdf12b0a98 . "./code.idejs/ide-dev.fasl")
(#x01277b666c1090fc49a593ddb589876c . "./code.idejs/ide.fasl")
(#x8ad12fa6fb77a9b3fa8d0727823a20e5 . "./code.idejs/ide-trace.fasl")
(#xd13078034488262ef25e3e73a60907a9 . "./code.idejs/ide-apropos.fasl")
(#x9f895886ff6883cc6db9a933aeabe18d . "./code.idejs/ide-inspector.fasl")
(#xc1ee21d2c62276bccda212c9f9b679b5 . "./code.idejs/ide-acache.fasl")
(#xb9ff7bc459ca4a256006d3705e83e4bc . "./code.idejs/ide-project.fasl")
(#xc717699580ff62eba160538ebb7bdef5 . "./code.idejs/ide-window-list.fasl")
(#xe06c3e52588a546760ef7cb8d72968a8 . "./code.idejs/ide-intro.fasl")
(#xc8c65d1f472d5abd8a03c61a9f219df8 . "./code.idejs/ide-clipboard.fasl")
(#xb10f7b0ac9183ace8e5a61a3a5f9437d . "./code.idejs/ide-editor.fasl")
(#xd161a2ad4b088da1e83de712a8304564 . "./code.idejs/ide-process-browser.fasl")
(#x21ce2d3d6791a1afee7c360a78fe2fda . "./code.idejs/ide-patch-dialog.fasl")
(#x6f170e2b58dd014bf55ed618a9eb9c7a . "./code.idejs/ide-navigator.fasl")
(#x29c4a4cb691acde2a2fe4116aa720ac8 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
